@@ -2,7 +2,6 @@
 
 #include "ifilter.h"
 #include "i18n.h"
-#include "igl.h"
 #include "icameraview.h"
 #include "iscenegraphfactory.h"
 #include "irendersystemfactory.h"
@@ -11,6 +10,8 @@
 #include "math/AABB.h"
 #include "util/ScopedBoolLock.h"
 #include "registry/registry.h"
+#include "render/CamRenderer.h"
+#include "render/SceneRenderWalker.h"
 
 #include "../GLWidget.h"
 #include <wx/sizer.h>
@@ -33,7 +34,9 @@ namespace
     const std::string PAUSE_BUTTON("pauseButton");
     const std::string STOP_BUTTON("stopButton");
 
-	const std::string RKEY_RENDERPREVIEW_SHOWGRID("ui/renderPreview/showGrid");
+	const std::string RKEY_RENDERPREVIEW_SHOWGRID("user/ui/renderPreview/showGrid");
+	const std::string RKEY_RENDERPREVIEW_FONTSIZE("user/ui/renderPreview/fontSize");
+	const std::string RKEY_RENDERPREVIEW_FONTSTYLE("user/ui/renderPreview/fontStyle");
 }
 
 RenderPreview::RenderPreview(wxWindow* parent, bool enableAnimation) :
@@ -42,7 +45,6 @@ RenderPreview::RenderPreview(wxWindow* parent, bool enableAnimation) :
     _initialised(false),
 	_renderGrid(registry::getValue<bool>(RKEY_RENDERPREVIEW_SHOWGRID)),
     _renderSystem(GlobalRenderSystemFactory().createRenderSystem()),
-    _sceneWalker(_renderer, _volumeTest),
     _viewOrigin(0, 0, 0),
     _viewAngles(0, 0, 0),
     _modelView(Matrix4::getIdentity()),
@@ -56,7 +58,7 @@ RenderPreview::RenderPreview(wxWindow* parent, bool enableAnimation) :
     _filtersMenu(GlobalUIManager().createFilterMenu()),
 	_filterTool(nullptr)
 {
-	Connect(wxEVT_TIMER, wxTimerEventHandler(RenderPreview::_onFrame), NULL, this);
+	Bind(wxEVT_TIMER, &RenderPreview::_onFrame, this);
 
     // Insert GL widget
 	_mainPanel->GetSizer()->Prepend(_glWidget, 1, wxEXPAND);
@@ -430,6 +432,14 @@ bool RenderPreview::drawPreview()
 
     util::ScopedBoolLock lock(_renderingInProgress);
 
+    if (!_glFont)
+    {
+        auto fontSize = registry::getValue<int>(RKEY_RENDERPREVIEW_FONTSIZE);
+        auto fontStyle = registry::getValue<std::string>(RKEY_RENDERPREVIEW_FONTSTYLE) == "Sans" ? 
+            IGLFont::Style::Sans : IGLFont::Style::Mono;
+        _glFont = GlobalOpenGL().getFont(fontStyle, fontSize);
+    }
+
     glViewport(0, 0, _previewWidth, _previewHeight);
 
     // Set up the render and clear the drawing area in any case
@@ -474,14 +484,14 @@ bool RenderPreview::drawPreview()
 	}
 
 	// Front-end render phase, collect OpenGLRenderable objects from the scene
-    getScene()->foreachVisibleNodeInVolume(_volumeTest, _sceneWalker);
+    render::CamRenderer renderer(_volumeTest);
+    render::SceneRenderWalker sceneWalker(renderer, _volumeTest);
+    getScene()->foreachVisibleNodeInVolume(_volumeTest, sceneWalker);
 
     RenderStateFlags flags = getRenderFlagsFill();
 
-    // Hack-inject the model rotation matrix just before the render pass
-    //Matrix4 modelView = _volumeTest.GetModelview().getMultipliedBy(_modelRotation);
-
     // Launch the back end rendering
+    renderer.submitToShaders();
     _renderSystem->render(flags, _volumeTest.GetModelview(), projection);
 
     // Give subclasses an opportunity to render their own on-screen stuff
@@ -501,7 +511,9 @@ void RenderPreview::renderWireFrame()
     Matrix4 projection = getProjectionMatrix(0.1f, 10000, PREVIEW_FOV, _previewWidth, _previewHeight);
 
     // Front-end render phase, collect OpenGLRenderable objects from the scene
-    getScene()->foreachVisibleNodeInVolume(_volumeTest, _sceneWalker);
+    render::CamRenderer renderer(_volumeTest);
+    render::SceneRenderWalker sceneWalker(renderer, _volumeTest);
+    getScene()->foreachVisibleNodeInVolume(_volumeTest, sceneWalker);
 
     // Launch the back end rendering
     _renderSystem->render(flags, _volumeTest.GetModelview(), projection);
@@ -782,7 +794,7 @@ void RenderPreview::drawTime()
 
     glRasterPos3f(1.0f, static_cast<float>(_previewHeight) - 1.0f, 0.0f);
 
-    GlobalOpenGL().drawString(fmt::format("{0:.3f} sec.", (_renderSystem->getTime() * 0.001f)));
+    _glFont->drawString(fmt::format("{0:.3f} sec.", (_renderSystem->getTime() * 0.001f)));
 }
 
 void RenderPreview::onGLKeyPress(wxKeyEvent& ev)
