@@ -1,7 +1,9 @@
 #pragma once
 
 #include <wx/app.h>
-#include "modulesystem/ApplicationContextImpl.h"
+#include <sigc++/connection.h>
+#include "module/ApplicationContextBase.h"
+#include "module/CoreModule.h"
 
 /**
 * Main application class required by wxWidgets
@@ -18,8 +20,12 @@ class RadiantApp :
 	public wxApp
 {
 	// The RadiantApp owns the ApplicationContext which is then passed to the
-	// ModuleRegistry as a refernce.
-	radiant::ApplicationContextImpl _context;
+	// ModuleRegistry as a reference.
+	radiant::ApplicationContextBase _context;
+
+	std::unique_ptr<module::CoreModule> _coreModule;
+
+	sigc::connection _modulesUnloadingHandler;
 
 public:
 	bool OnInit() override;
@@ -32,4 +38,5 @@ public:
 
 private:
 	void onStartupEvent(wxCommandEvent& ev);
+	void onModulesUnloading();
 };
