@@ -35,6 +35,7 @@
 #include <functional>
 #include <sigc++/retype_return.h>
 #include "render/VectorLightList.h"
+#include "render/CamRenderer.h"
 
 namespace ui
 {
@@ -548,242 +549,6 @@ bool CamWnd::freeMoveEnabled() const
     return _freeMoveEnabled;
 }
 
-namespace
-{
-
-// Implementation of RenderableCollector for the 3D camera view.
-class CamRenderer: public RenderableCollector
-{
-    // The View object for object culling
-    const render::View& _view;
-
-    // Render statistics
-    render::RenderStatistics& _renderStats;
-
-    // Highlight state
-    bool _highlightFaces = false;
-    bool _highlightPrimitives = false;
-    Shader& _highlightedPrimitiveShader;
-    Shader& _highlightedFaceShader;
-
-    // All lights we have received from the scene
-    std::list<const RendererLight*> _sceneLights;
-
-    // Legacy lit renderable which provided its own LightSources to
-    // addRenderable()
-    struct LegacyLitRenderable
-    {
-        const OpenGLRenderable& renderable;
-        const LightSources* lights = nullptr;
-        Matrix4 local2World;
-        const IRenderEntity* entity = nullptr;
-
-        LegacyLitRenderable(const OpenGLRenderable& r, const LightSources* l,
-                            const Matrix4& l2w, const IRenderEntity* e)
-        : renderable(r), lights(l), local2World(l2w), entity(e)
-        {}
-    };
-    using LegacyLitRenderables = std::vector<LegacyLitRenderable>;
-
-    // Legacy renderables with their own light lists. No processing needed;
-    // just store them until it's time to submit to shaders.
-    using LegacyRenderablesByShader = std::map<Shader*, LegacyLitRenderables>;
-    LegacyRenderablesByShader _legacyRenderables;
-
-    // Lit renderable provided via addLitRenderable(), for which we construct
-    // the light list with lights received via addLight().
-    struct LitRenderable
-    {
-        // Renderable information submitted with addLitObject()
-        const OpenGLRenderable& renderable;
-        const LitObject& litObject;
-        Matrix4 local2World;
-        const IRenderEntity* entity = nullptr;
-
-        // Calculated list of intersecting lights (initially empty)
-        render::lib::VectorLightList lights;
-    };
-    using LitRenderables = std::vector<LitRenderable>;
-
-    // Renderables added with addLitObject() need to be stored until their
-    // light lists can be calculated, which can't happen until all the lights
-    // are submitted too.
-    std::map<Shader*, LitRenderables> _litRenderables;
-
-    // Intersect all received renderables wiith lights
-    void calculateLightIntersections()
-    {
-        // For each shader
-        for (auto i = _litRenderables.begin(); i != _litRenderables.end(); ++i)
-        {
-            // For each renderable associated with this shader
-            for (auto j = i->second.begin(); j != i->second.end(); ++j)
-            {
-                // Test intersection between the LitObject and each light in
-                // the scene
-                for (const RendererLight* l: _sceneLights)
-                {
-                    if (j->litObject.intersectsLight(*l))
-                        j->lights.addLight(*l);
-                }
-            }
-        }
-    }
-
-public:
-
-    // Initialise CamRenderer with the highlight shaders
-    CamRenderer(const render::View& view, Shader& primHighlightShader,
-                Shader& faceHighlightShader, render::RenderStatistics& stats)
-    : _view(view),
-      _renderStats(stats),
-      _highlightedPrimitiveShader(primHighlightShader),
-      _highlightedFaceShader(faceHighlightShader)
-    {}
-
-    // Instruct the CamRenderer to push its sorted renderables to their
-    // respective shaders and perform the actual render
-    void backendRender(RenderStateFlags allowedFlags, const Matrix4& modelView,
-                       const Matrix4& projection)
-    {
-        // Calculate intersections between lights and renderables we have received
-        calculateLightIntersections();
-
-        // For each shader in the map
-        for (auto i = _legacyRenderables.begin();
-             i != _legacyRenderables.end();
-             ++i)
-        {
-            // Iterate over the list of renderables for this shader, submitting
-            // each one
-            Shader* shader = i->first;
-            wxASSERT(shader);
-            for (auto j = i->second.begin(); j != i->second.end(); ++j)
-            {
-                const LegacyLitRenderable& lr = *j;
-                shader->addRenderable(lr.renderable, lr.local2World,
-                                      lr.lights, lr.entity);
-            }
-        }
-
-        // Tell the render system to render its shaders and renderables
-        GlobalRenderSystem().render(allowedFlags, modelView, projection,
-                                    _view.getViewer());
-    }
-
-    // RenderableCollector implementation
-
-    bool supportsFullMaterials() const override { return true; }
-
-    void setHighlightFlag(Highlight::Flags flags, bool enabled) override
-    {
-        if (flags & Highlight::Faces)
-        {
-            _highlightFaces = enabled;
-        }
-
-        if (flags & Highlight::Primitives)
-        {
-            _highlightPrimitives = enabled;
-        }
-    }
-
-    void addLight(const RendererLight& light) override
-    {
-        // Determine if this light is visible within the view frustum
-        VolumeIntersectionValue viv = _view.TestAABB(light.lightAABB());
-        if (viv == VOLUME_OUTSIDE)
-        {
-            // Not interested
-            _renderStats.addLight(false);
-        }
-        else
-        {
-            // Store the light in our list of scene lights
-            _sceneLights.push_back(&light);
-
-            // Count the light for the stats display
-            _renderStats.addLight(true);
-        }
-    }
-
-    void addRenderable(Shader& shader, const OpenGLRenderable& renderable,
-                       const Matrix4& world, const LightSources* lights,
-                       const IRenderEntity* entity) override
-    {
-        if (_highlightPrimitives)
-            _highlightedPrimitiveShader.addRenderable(renderable, world,
-                                                      lights, entity);
-
-        if (_highlightFaces)
-            _highlightedFaceShader.addRenderable(renderable, world,
-                                                 lights, entity);
-
-        // Construct an entry for this shader in the map if it is the first
-        // time we've seen it
-        auto iter = _legacyRenderables.find(&shader);
-        if (iter == _legacyRenderables.end())
-        {
-            // Add an entry for this shader, and pre-allocate some space in the
-            // vector to avoid too many expansions during scenegraph traversal.
-            LegacyLitRenderables emptyList;
-            emptyList.reserve(1024);
-
-            auto result = _legacyRenderables.insert(
-                std::make_pair(&shader, std::move(emptyList))
-            );
-            wxASSERT(result.second);
-            iter = result.first;
-        }
-        wxASSERT(iter != _legacyRenderables.end());
-
-        // Add the renderable and its lights to the list of lit renderables for
-        // this shader
-        wxASSERT(iter->first == &shader);
-        iter->second.emplace_back(renderable, lights, world, entity);
-    }
-
-    void addLitRenderable(Shader& shader,
-                          OpenGLRenderable& renderable,
-                          const Matrix4& localToWorld,
-                          const LitObject& litObject,
-                          const IRenderEntity* entity = nullptr) override
-    {
-        if (_highlightPrimitives)
-            _highlightedPrimitiveShader.addRenderable(renderable, localToWorld,
-                                                      nullptr, entity);
-
-        if (_highlightFaces)
-            _highlightedFaceShader.addRenderable(renderable, localToWorld,
-                                                 nullptr, entity);
-
-        // Construct an entry for this shader in the map if it is the first
-        // time we've seen it
-        auto iter = _litRenderables.find(&shader);
-        if (iter == _litRenderables.end())
-        {
-            // Add an entry for this shader, and pre-allocate some space in the
-            // vector to avoid too many expansions during scenegraph traversal.
-            LitRenderables emptyList;
-            emptyList.reserve(1024);
-
-            auto result = _litRenderables.insert(
-                std::make_pair(&shader, std::move(emptyList))
-            );
-            wxASSERT(result.second);
-            iter = result.first;
-        }
-        wxASSERT(iter != _litRenderables.end());
-        wxASSERT(iter->first == &shader);
-
-        // Store a LitRenderable object for this renderable
-        LitRenderable lr { renderable, litObject, localToWorld, entity };
-        iter->second.push_back(std::move(lr));
-    }
-};
-
-}
-
 void CamWnd::performFreeMove(int dx, int dy)
 {
     int angleSpeed = getCameraSettings()->angleSpeed();
@@ -804,7 +569,7 @@ void CamWnd::performFreeMove(int dx, int dy)
             origin += _camera->getForwardVector() * strafespeed * dy * forwardStrafeFactor;
         }
         else {
-            origin += _camera->getForwardVector() * strafespeed * dy;
+            origin += _camera->getUpVector() * strafespeed * dy;
         }
     }
     else // free rotation
@@ -835,6 +600,14 @@ void CamWnd::onDeferredMotionDelta(int x, int y)
     queueDraw();
 }
 
+void CamWnd::ensureFont()
+{
+    if (!_glFont)
+    {
+        _glFont = GlobalOpenGL().getFont(getCameraSettings()->fontStyle(), getCameraSettings()->fontSize());
+    }
+}
+
 void CamWnd::Cam_Draw()
 {
     wxSize glSize = _wxGLWidget->GetSize();
@@ -852,6 +625,8 @@ void CamWnd::Cam_Draw()
         return; // otherwise we'll receive OpenGL errors in ortho rendering below
     }
 
+    ensureFont();
+
     glViewport(0, 0, width, height);
 
     // enable depth buffer writes
@@ -967,9 +742,13 @@ void CamWnd::Cam_Draw()
     // Main scene render
     {
         // Front end (renderable collection from scene)
-        CamRenderer renderer(_view, *_primitiveHighlightShader,
-                             *_faceHighlightShader, _renderStats);
+        render::CamRenderer renderer(_view, _primitiveHighlightShader.get(),
+                                     _faceHighlightShader.get());
         render::RenderableCollectionWalker::CollectRenderablesInScene(renderer, _view);
+
+        // Accumulate render statistics
+        _renderStats.setLightCount(renderer.getVisibleLights(),
+                                   renderer.getTotalLights());
         _renderStats.frontEndComplete();
 
         // Render any active mousetools
@@ -978,9 +757,12 @@ void CamWnd::Cam_Draw()
             i.second->render(GlobalRenderSystem(), renderer, _view);
         }
 
-        // Backend (shader rendering)
-        renderer.backendRender(allowedRenderFlags, _camera->getModelView(),
-                               _camera->getProjection());
+        // Back end (submit to shaders and do the actual render)
+        renderer.submitToShaders(
+            getCameraSettings()->getRenderMode() == RENDER_MODE_LIGHTING
+        );
+        GlobalRenderSystem().render(allowedRenderFlags, _camera->getModelView(),
+                                    _camera->getProjection(), _view.getViewer());
     }
 
     // greebo: Draw the clipper's points (skipping the depth-test)
@@ -1051,7 +833,7 @@ void CamWnd::Cam_Draw()
     if (!statString.empty())
         statString += " | ";
     statString += _renderStats.getStatString();
-    GlobalOpenGL().drawString(statString);
+    _glFont->drawString(statString);
 
     drawTime();
 
@@ -1430,7 +1212,7 @@ void CamWnd::drawTime()
     glOrtho(0, static_cast<float>(width), 0, static_cast<float>(height), -100, 100);
 
     glScalef(1, -1, 1);
-    glTranslatef(static_cast<float>(width) - 90, -static_cast<float>(height), 0);
+    glTranslatef(static_cast<float>(width) - 105, -static_cast<float>(height), 0);
 
     glMatrixMode(GL_MODELVIEW);
     glLoadIdentity();
@@ -1453,10 +1235,10 @@ void CamWnd::drawTime()
     glColor3f(1.f, 1.f, 1.f);
     glLineWidth(1);
 
-    glRasterPos3f(1.0f, static_cast<float>(height) - 1.0f, 0.0f);
+    glRasterPos3f(1.0f, static_cast<float>(height) - 4.0f, 0.0f);
 
     std::size_t time = GlobalRenderSystem().getTime();
-    GlobalOpenGL().drawString(fmt::format("Time: {0:.3f} sec.", (time * 0.001f)));
+    _glFont->drawString(fmt::format("Time: {0:.3f} sec.", (time * 0.001f)));
 }
 
 void CamWnd::handleFreeMoveKeyEvent(KeyEventType eventType, unsigned int movementFlags)
