@@ -74,7 +74,7 @@ void SkinChooser::populateWindow()
     splitter->SetMinimumPaneSize(10); // disallow unsplitting
 
 	// Create the tree view
-    _treeView = wxutil::TreeView::CreateWithModel(splitter, _treeStore, wxDV_NO_HEADER);
+    _treeView = wxutil::TreeView::CreateWithModel(splitter, _treeStore.get(), wxDV_NO_HEADER);
 	_treeView->SetMinClientSize(wxSize(GetSize().GetWidth() / 5, -1));
 	
 	// Single column to display the skin name
@@ -182,7 +182,7 @@ void SkinChooser::populateSkins()
 	for (const auto& skin : skins)
 	{
         pop.addPath(skin, [&] (wxutil::TreeModel::Row& row, 
-            const std::string& leafName, bool isFolder)
+            const std::string& path, const std::string& leafName, bool isFolder)
         {
             // Get the display path, everything after rightmost slash
             std::string displayPath = leafName.substr(leafName.rfind("/") + 1);
