@@ -7,6 +7,7 @@
 #include "igroupdialog.h"
 #include "ipreferencesystem.h"
 #include "ishaders.h"
+#include "ishaderclipboard.h"
 #include "ieventmanager.h"
 
 #include "wxutil/MultiMonitor.h"
@@ -19,22 +20,22 @@
 #include <wx/artprov.h>
 #include <wx/sizer.h>
 #include <wx/radiobut.h>
+#include <wx/frame.h>
 
 #include <iostream>
 #include <map>
 
 #include "registry/registry.h"
 #include "shaderlib.h"
-#include "selection/algorithm/Shader.h"
-#include "selection/shaderclipboard/ShaderClipboard.h"
 #include "string/string.h"
+#include "util/ScopedBoolLock.h"
 #include "ui/texturebrowser/TextureBrowser.h"
-#include "ui/common/ShaderDefinitionView.h"
+#include "ui/common/MaterialDefinitionView.h"
 #include "ui/mainframe/ScreenUpdateBlocker.h"
 #include "ui/common/TexturePreviewCombo.h"
 
 #include "debugging/ScopedDebugTimer.h"
-#include "modulesystem/StaticModule.h"
+#include "module/StaticModule.h"
 
 #include <functional>
 #include "string/predicate.h"
@@ -397,7 +398,8 @@ MediaBrowser::MediaBrowser() :
 	_mode(TreeMode::ShowAll),
 	_favourites(new Favourites),
 	_preview(nullptr),
-	_isPopulated(false)
+	_isPopulated(false),
+	_blockShaderClipboardUpdates(false)
 {}
 
 void MediaBrowser::construct()
@@ -471,7 +473,7 @@ void MediaBrowser::construct()
 		if (!isDirectorySelected() && !selection.empty())
 		{
 			// Pass shader name to the selection system
-			selection::algorithm::applyShaderToSelection(selection);
+			GlobalCommandSystem().executeCommand("SetShaderOnSelection", selection);
 		}
 	});
 
@@ -533,6 +535,9 @@ void MediaBrowser::construct()
 		if (ev.GetEventObject() == _mainWidget)
 		{
 			_treeView = nullptr;
+			_shaderClipboardConn.disconnect();
+			_materialDefsLoaded.disconnect();
+			_materialDefsUnloaded.disconnect();
 		}
 		ev.Skip();
 	});
@@ -562,7 +567,7 @@ bool MediaBrowser::isFavouriteSelected()
 	return row[_columns.isFavourite].getBool();
 }
 
-void MediaBrowser::onRadiantStartup()
+void MediaBrowser::onMainFrameConstructed()
 {
 	// Add the Media Browser page
 	IGroupDialog::PagePtr mediaBrowserPage(new IGroupDialog::Page);
@@ -816,7 +821,7 @@ bool MediaBrowser::_testLoadInTexView()
 void MediaBrowser::_onApplyToSel()
 {
 	// Pass shader name to the selection system
-	selection::algorithm::applyShaderToSelection(getSelection());
+	GlobalCommandSystem().executeCommand("SetShaderOnSelection", getSelection());
 }
 
 // Check if a single non-directory texture is selected (used by multiple menu
@@ -834,7 +839,9 @@ void MediaBrowser::_onShowShaderDefinition()
 	std::string shaderName = getSelection();
 
 	// Construct a shader view and pass the shader name
-	ShaderDefinitionView::ShowDialog(shaderName);
+	auto view = new MaterialDefinitionView(shaderName);
+	view->ShowModal();
+	view->Destroy();
 }
 
 void MediaBrowser::_onSelectItems(bool select)
@@ -843,11 +850,11 @@ void MediaBrowser::_onSelectItems(bool select)
 
     if (select)
     {
-        selection::algorithm::selectItemsByShader(shaderName);
+		GlobalCommandSystem().executeCommand("SelectItemsByShader", shaderName);
     }
     else
     {
-        selection::algorithm::deselectItemsByShader(shaderName);
+		GlobalCommandSystem().executeCommand("DeselectItemsByShader", shaderName);
     }
 }
 
@@ -943,11 +950,13 @@ void MediaBrowser::_onExpose(wxPaintEvent& ev)
 
 void MediaBrowser::handleSelectionChange()
 {
+	util::ScopedBoolLock lock(_blockShaderClipboardUpdates);
+
 	// Update the preview if a texture is selected
 	if (!isDirectorySelected())
 	{
 		_preview->SetTexture(getSelection());
-		GlobalShaderClipboard().setSource(getSelection());
+		GlobalShaderClipboard().setSourceShader(getSelection());
 	}
 	else
 	{
@@ -983,17 +992,18 @@ const StringSet& MediaBrowser::getDependencies() const
 		_dependencies.insert(MODULE_EVENTMANAGER);
 		_dependencies.insert(MODULE_SHADERSYSTEM);
 		_dependencies.insert(MODULE_UIMANAGER);
+		_dependencies.insert(MODULE_SHADERCLIPBOARD);
+		_dependencies.insert(MODULE_MAINFRAME);
 	}
 
 	return _dependencies;
 }
 
-void MediaBrowser::initialiseModule(const ApplicationContext& ctx)
+void MediaBrowser::initialiseModule(const IApplicationContext& ctx)
 {
 	rMessage() << getName() << "::initialiseModule called." << std::endl;
 
 	GlobalCommandSystem().addCommand("ToggleMediaBrowser", sigc::mem_fun(this, &MediaBrowser::togglePage));
-	GlobalEventManager().addCommand("ToggleMediaBrowser", "ToggleMediaBrowser");
 
 	// We need to create the liststore and widgets before attaching ourselves
 	// to the material manager as observer, as the attach() call below
@@ -1001,8 +1011,8 @@ void MediaBrowser::initialiseModule(const ApplicationContext& ctx)
 	construct();
 
 	// The startup event will add this page to the group dialog tab
-	GlobalRadiant().signal_radiantStarted().connect(
-		sigc::mem_fun(*this, &MediaBrowser::onRadiantStartup)
+	GlobalMainFrame().signal_MainFrameConstructed().connect(
+		sigc::mem_fun(*this, &MediaBrowser::onMainFrameConstructed)
 	);
 
 	// Attach to the MaterialManager to get notified on unrealise/realise
@@ -1017,15 +1027,30 @@ void MediaBrowser::initialiseModule(const ApplicationContext& ctx)
 
 	// Start loading materials
 	populate();
+
+	_shaderClipboardConn = GlobalShaderClipboard().signal_sourceChanged().connect(
+		sigc::mem_fun(this, &MediaBrowser::onShaderClipboardSourceChanged)
+	);
 }
 
 void MediaBrowser::shutdownModule()
 {
+	_shaderClipboardConn.disconnect();
 	_emptyFavouritesLabel = wxDataViewItem();
 	_materialDefsLoaded.disconnect();
 	_materialDefsUnloaded.disconnect();
 }
 
+void MediaBrowser::onShaderClipboardSourceChanged()
+{
+	if (_blockShaderClipboardUpdates)
+	{
+		return;
+	}
+
+	setSelection(GlobalShaderClipboard().getShaderName());
+}
+
 // Static module
 module::StaticModule<MediaBrowser> mediaBrowserModule;
 
