@@ -7,6 +7,7 @@
 #include "itextstream.h"
 #include "iregistry.h"
 #include "imainframe.h"
+#include "imodelcache.h"
 #include "imodel.h"
 #include "modelskin.h"
 #include "imodelcache.h"
@@ -27,8 +28,6 @@
 
 #include <functional>
 
-#include "map/algorithm/Models.h"
-
 namespace ui
 {
 
@@ -156,9 +155,9 @@ ModelSelector& ModelSelector::Instance()
         // Not yet instantiated, do it now
         instancePtr.reset(new ModelSelector);
 
-        // Register this instance with GlobalRadiant() at once
-        GlobalRadiant().signal_radiantShutdown().connect(
-            sigc::mem_fun(*instancePtr, &ModelSelector::onRadiantShutdown)
+        // Pre-destruction cleanup
+        GlobalMainFrame().signal_MainFrameShuttingDown().connect(
+            sigc::mem_fun(*instancePtr, &ModelSelector::onMainFrameShuttingDown)
         );
     }
 
@@ -171,7 +170,7 @@ ModelSelectorPtr& ModelSelector::InstancePtr()
     return _instancePtr;
 }
 
-void ModelSelector::onRadiantShutdown()
+void ModelSelector::onMainFrameShuttingDown()
 {
     rMessage() << "ModelSelector shutting down." << std::endl;
 
@@ -484,7 +483,7 @@ void ModelSelector::onReloadModels(wxCommandEvent& ev)
 	_preselectedModel = getSelectedValue(_columns.vfspath);
 
 	// This will fire the models reloaded signal after some time
-	map::algorithm::refreshModels(false);
+	GlobalModelCache().refreshModels(false);
 }
 
 void ModelSelector::onReloadSkins(wxCommandEvent& ev)
