@@ -1,6 +1,7 @@
 #include "EClassManager.h"
 
 #include "iarchive.h"
+#include "ieclasscolours.h"
 #include "i18n.h"
 #include "iregistry.h"
 #include "icommandsystem.h"
@@ -64,6 +65,9 @@ IEntityClassPtr EClassManager::findOrInsert(const std::string& name, bool has_br
     // so that brushes of unknown entites don't get lost (issue #240)
     eclass = Doom3EntityClass::create(lName, true);
 
+    // Any overrides should also apply to entityDefs that are crated on the fly
+    GlobalEclassColourManager().applyColours(eclass);
+
     // Try to insert the class
     return insertUnique(eclass);
 }
@@ -172,8 +176,6 @@ void EClassManager::resolveInheritance()
 
 void EClassManager::ensureDefsLoaded()
 {
-    assert(_realised);
-
     _defLoader.ensureFinished();
 }
 
@@ -181,10 +183,25 @@ void EClassManager::loadDefAndResolveInheritance()
 {
     parseDefFiles();
     resolveInheritance();
+    applyColours();
 
 	_defsLoadedSignal.emit();
 }
 
+void EClassManager::applyColours()
+{
+    GlobalEclassColourManager().foreachOverrideColour([&](const std::string& eclass, const Vector3& colour)
+    {
+        auto foundEclass = _entityClasses.find(string::to_lower_copy(eclass));
+
+        if (foundEclass != _entityClasses.end())
+        {
+            rDebug() << "Applying colour " << colour << " to eclass " << eclass << std::endl;
+            foundEclass->second->setColour(colour);
+        }
+    });
+}
+
 void EClassManager::realise()
 {
 	if (_realised) 
@@ -280,6 +297,7 @@ const StringSet& EClassManager::getDependencies() const
 		_dependencies.insert(MODULE_VIRTUALFILESYSTEM);
 		_dependencies.insert(MODULE_XMLREGISTRY);
 		_dependencies.insert(MODULE_COMMANDSYSTEM);
+		_dependencies.insert(MODULE_ECLASS_COLOUR_MANAGER);
 	}
 
 	return _dependencies;
@@ -290,15 +308,24 @@ void EClassManager::initialiseModule(const IApplicationContext& ctx)
 	rMessage() << "EntityClassDoom3::initialiseModule called." << std::endl;
 
 	GlobalFileSystem().addObserver(*this);
-	realise();
+
+    if (GlobalFileSystem().isInitialised())
+    {
+        realise();
+    }
 
 	GlobalCommandSystem().addCommand("ReloadDefs", std::bind(&EClassManager::reloadDefsCmd, this, std::placeholders::_1));
+
+    _eclassColoursChanged = GlobalEclassColourManager().sig_overrideColourChanged().connect(
+        sigc::mem_fun(this, &EClassManager::onEclassOverrideColourChanged));
 }
 
 void EClassManager::shutdownModule()
 {
 	rMessage() << "EntityClassDoom3::shutdownModule called." << std::endl;
 
+    _eclassColoursChanged.disconnect();
+
 	GlobalFileSystem().removeObserver(*this);
 
 	// Unrealise ourselves and wait for threads to finish
@@ -312,6 +339,29 @@ void EClassManager::shutdownModule()
 	_models.clear();
 }
 
+void EClassManager::onEclassOverrideColourChanged(const std::string& eclass, bool overrideRemoved)
+{
+    // An override colour in the IColourManager instance has changed
+    // Do we have an affected eclass with that name?
+    auto foundEclass = _entityClasses.find(eclass);
+
+    if (foundEclass == _entityClasses.end())
+    {
+        return;
+    }
+
+    // If the override was removed, we just reset the colour
+    // We perform this switch to avoid firing the eclass changed signal twice
+    if (overrideRemoved)
+    {
+        foundEclass->second->resetColour();
+    }
+    else
+    {
+        GlobalEclassColourManager().applyColours(foundEclass->second);
+    }
+}
+
 // This takes care of relading the entityDefs and refreshing the scenegraph
 void EClassManager::reloadDefsCmd(const cmd::ArgumentList& args)
 {
