@@ -1,9 +1,12 @@
 #include "LightNode.h"
 
 #include "itextstream.h"
+#include "icolourscheme.h"
 #include "../EntitySettings.h"
 #include <functional>
 
+#include "registry/CachedKey.h"
+
 namespace entity {
 
 // --------- LightNode implementation ------------------------------------
@@ -14,14 +17,16 @@ LightNode::LightNode(const IEntityClassPtr& eclass) :
 		   *this,
 		   Callback(std::bind(&scene::Node::transformChanged, this)),
 		   Callback(std::bind(&scene::Node::boundsChanged, this)),
-		   Callback(std::bind(&LightNode::lightChanged, this))),
+		   Callback(std::bind(&LightNode::onLightRadiusChanged, this))),
 	_lightCenterInstance(_light.getDoom3Radius().m_centerTransformed, std::bind(&LightNode::selectedChangedComponent, this, std::placeholders::_1)),
 	_lightTargetInstance(_light.targetTransformed(), std::bind(&LightNode::selectedChangedComponent, this, std::placeholders::_1)),
 	_lightRightInstance(_light.rightTransformed(), _light.targetTransformed(), std::bind(&LightNode::selectedChangedComponent, this, std::placeholders::_1)),
 	_lightUpInstance(_light.upTransformed(), _light.targetTransformed(), std::bind(&LightNode::selectedChangedComponent, this, std::placeholders::_1)),
 	_lightStartInstance(_light.startTransformed(), std::bind(&LightNode::selectedChangedComponent, this, std::placeholders::_1)),
 	_lightEndInstance(_light.endTransformed(), std::bind(&LightNode::selectedChangedComponent, this, std::placeholders::_1)),
-	_dragPlanes(std::bind(&LightNode::selectedChangedComponent, this, std::placeholders::_1))
+	_dragPlanes(std::bind(&LightNode::selectedChangedComponent, this, std::placeholders::_1)),
+    _renderableRadius(_light._lightBox.origin),
+    _renderableFrustum(_light._lightBox.origin, _light._lightStartTransformed, _light._frustum)
 {}
 
 LightNode::LightNode(const LightNode& other) :
@@ -32,14 +37,16 @@ LightNode::LightNode(const LightNode& other) :
            _entity,
            Callback(std::bind(&Node::transformChanged, this)),
 		   Callback(std::bind(&Node::boundsChanged, this)),
-		   Callback(std::bind(&LightNode::lightChanged, this))),
+		   Callback(std::bind(&LightNode::onLightRadiusChanged, this))),
 	_lightCenterInstance(_light.getDoom3Radius().m_centerTransformed, std::bind(&LightNode::selectedChangedComponent, this, std::placeholders::_1)),
 	_lightTargetInstance(_light.targetTransformed(), std::bind(&LightNode::selectedChangedComponent, this, std::placeholders::_1)),
 	_lightRightInstance(_light.rightTransformed(), _light.targetTransformed(), std::bind(&LightNode::selectedChangedComponent, this, std::placeholders::_1)),
 	_lightUpInstance(_light.upTransformed(), _light.targetTransformed(), std::bind(&LightNode::selectedChangedComponent, this, std::placeholders::_1)),
 	_lightStartInstance(_light.startTransformed(), std::bind(&LightNode::selectedChangedComponent, this, std::placeholders::_1)),
 	_lightEndInstance(_light.endTransformed(), std::bind(&LightNode::selectedChangedComponent, this,std::placeholders:: _1)),
-	_dragPlanes(std::bind(&LightNode::selectedChangedComponent, this, std::placeholders::_1))
+	_dragPlanes(std::bind(&LightNode::selectedChangedComponent, this, std::placeholders::_1)),
+    _renderableRadius(_light._lightBox.origin),
+    _renderableFrustum(_light._lightBox.origin, _light._lightStartTransformed, _light._frustum)
 {}
 
 LightNodePtr LightNode::Create(const IEntityClassPtr& eclass)
@@ -71,8 +78,10 @@ AABB LightNode::getSelectAABB() const
 	return returnValue;
 }
 
-void LightNode::lightChanged() {
-	GlobalRenderSystem().lightChanged();
+void LightNode::onLightRadiusChanged()
+{
+    // Light radius changed, mark bounds as dirty
+    boundsChanged();
 }
 
 const AABB& LightNode::localAABB() const {
@@ -88,8 +97,6 @@ void LightNode::onInsertIntoScene(scene::IMapRootNode& root)
 {
 	// Call the base class first
 	EntityNode::onInsertIntoScene(root);
-
-	GlobalRenderSystem().attachLight(_light);
 }
 
 void LightNode::onRemoveFromScene(scene::IMapRootNode& root)
@@ -97,8 +104,6 @@ void LightNode::onRemoveFromScene(scene::IMapRootNode& root)
 	// Call the base class first
 	EntityNode::onRemoveFromScene(root);
 
-	GlobalRenderSystem().detachLight(_light);
-
 	// De-select all child components as well
 	setSelectedComponents(false, SelectionSystem::eVertex);
 	setSelectedComponents(false, SelectionSystem::eFace);
@@ -106,9 +111,17 @@ void LightNode::onRemoveFromScene(scene::IMapRootNode& root)
 
 void LightNode::testSelect(Selector& selector, SelectionTest& test)
 {
-	EntityNode::testSelect(selector, test);
+    // Generic entity selection
+    EntityNode::testSelect(selector, test);
 
-	_light.testSelect(selector, test, localToWorld());
+    // Light specific selection
+    test.BeginMesh(localToWorld());
+    SelectionIntersection best;
+    aabb_testselect(_light._lightBox, test, best);
+    if (best.isValid())
+    {
+        selector.addIntersection(best);
+    }
 }
 
 // greebo: Returns true if drag planes or one or more light vertices are selected
@@ -276,23 +289,62 @@ void LightNode::renderSolid(RenderableCollector& collector, const VolumeTest& vo
 
     // Re-use the same method as in wireframe rendering for the moment
     const bool lightIsSelected = isSelected();
-    _light.renderWireframe(
-        collector, volume, localToWorld(), lightIsSelected
-    );
+    renderLightVolume(collector, localToWorld(), lightIsSelected);
 
     renderInactiveComponents(collector, volume, lightIsSelected);
 }
 
 void LightNode::renderWireframe(RenderableCollector& collector, const VolumeTest& volume) const
 {
-	EntityNode::renderWireframe(collector, volume);
+    EntityNode::renderWireframe(collector, volume);
 
-	const bool lightIsSelected = isSelected();
-	_light.renderWireframe(
-		collector, volume, localToWorld(), lightIsSelected
-	);
+    const bool lightIsSelected = isSelected();
+    renderLightVolume(collector, localToWorld(), lightIsSelected);
 
-	renderInactiveComponents(collector, volume, lightIsSelected);
+    renderInactiveComponents(collector, volume, lightIsSelected);
+}
+
+void LightNode::renderLightVolume(RenderableCollector& collector,
+                                  const Matrix4& localToWorld,
+                                  bool selected) const
+{
+    // Obtain the appropriate Shader for the light volume colour
+    static registry::CachedKey<bool> _overrideColKey(
+        colours::RKEY_OVERRIDE_LIGHTCOL
+    );
+    Shader* colourShader = _overrideColKey.get() ? EntityNode::_wireShader.get()
+                                                 : _colourKey.getWireShader();
+    if (!colourShader)
+        return;
+
+    // Main render, submit the diamond that represents the light entity
+    collector.addRenderable(*colourShader, *this, localToWorld);
+
+    // Render bounding box if selected or the showAllLighRadii flag is set
+    if (selected || EntitySettings::InstancePtr()->getShowAllLightRadii())
+    {
+        if (_light.isProjected())
+        {
+            // greebo: This is not much of an performance impact as the
+            // projection gets only recalculated when it has actually changed.
+            _light.updateProjection();
+            collector.addRenderable(*colourShader, _renderableFrustum, localToWorld);
+        }
+        else
+        {
+            updateRenderableRadius();
+            collector.addRenderable(*colourShader, _renderableRadius, localToWorld);
+        }
+    }
+}
+
+/* greebo: Calculates the corners of the light radii box and rotates them according the rotation matrix.
+ */
+void LightNode::updateRenderableRadius() const
+{
+    // greebo: Don't rotate the light radius box, that's done via local2world
+    AABB lightbox(_light._lightBox.origin, _light.m_doom3Radius.m_radiusTransformed);
+    lightbox.getCorners(_renderableRadius.m_points);
 }
 
 void LightNode::setRenderSystem(const RenderSystemPtr& renderSystem)
@@ -392,6 +444,45 @@ void LightNode::renderInactiveComponents(RenderableCollector& collector, const V
 	}
 }
 
+// Backend render function (GL calls)
+void LightNode::render(const RenderInfo& info) const
+{
+    // Revert the light "diamond" to default extents for drawing
+    AABB tempAABB(_light._lightBox.origin, Vector3(8, 8, 8));
+
+    // Calculate the light vertices of this bounding box and store them into <points>
+    Vector3 max(tempAABB.origin + tempAABB.extents);
+    Vector3 min(tempAABB.origin - tempAABB.extents);
+    Vector3 mid(tempAABB.origin);
+
+    // top, bottom, tleft, tright, bright, bleft
+    Vector3 points[6] =
+    {
+        Vector3(mid[0], mid[1], max[2]),
+        Vector3(mid[0], mid[1], min[2]),
+        Vector3(min[0], max[1], mid[2]),
+        Vector3(max[0], max[1], mid[2]),
+        Vector3(max[0], min[1], mid[2]),
+        Vector3(min[0], min[1], mid[2])
+    };
+
+    // greebo: Draw the small cube representing the light origin.
+    typedef unsigned int index_t;
+    const index_t indices[24] = {
+        0, 2, 3,
+        0, 3, 4,
+        0, 4, 5,
+        0, 5, 2,
+        1, 2, 5,
+        1, 5, 4,
+        1, 4, 3,
+        1, 3, 2
+    };
+
+    glVertexPointer(3, GL_DOUBLE, 0, points);
+    glDrawElements(GL_TRIANGLES, sizeof(indices) / sizeof(index_t), RenderIndexTypeID, indices);
+}
+
 void LightNode::evaluateTransform()
 {
 	if (getType() == TRANSFORM_PRIMITIVE)
